<?php

namespace app\controller\admin;

use laytp\controller\Backend;
use think\facade\Config;
use laytp\library\CommonFun;

/**
 * 示例文本
 */
class Mode extends Backend
{

    /**
     * member模型对象
     * @var \app\model\Mode
     */
    protected $model;
	protected $hasSoftDel=1;//是否拥有软删除功能

    protected $noNeedLogin = []; // 无需登录即可请求的方法
    protected $noNeedAuth = ['index']; // 无需鉴权即可请求的方法

    public function _initialize()
    {
        $this->model = new \app\model\Mode();
    }


    //查看和搜索列表
    public function index(){
        global $_W;
        $where  = $this->buildSearchParams();
        $where[] = ['uniacid','=',$_W['uniacid']];
        $order  = $this->buildOrder();
        $data   = $this->model->where($where)->with(['img_file'])->order($order);
        $paging = $this->request->param('paging', false);
        if ($paging) {
            $limit        = $this->request->param('limit', Config::get('paginate.limit'));
            $data         = $data->paginate($limit)->toArray();
            $data['data'] = $this->getSelectedData($data['data']);
        } else {
            $data = $data->select()->toArray();
        }
        return $this->success('数据获取成功', $data);
    }

    //查看详情
    public function info()
    {
        $id   = $this->request->param('id');
        $info = $this->model->with(['img_file'])->find($id);
        return $this->success('获取成功', $info);
    }

    //设置
    public function setStatus()
    {
        $id       = $this->request->post('id');
        $fieldVal = $this->request->post('field_val');
        $isRecycle = $this->request->post('is_recycle');
        $update['status'] = $fieldVal;
        try {
            if($isRecycle) {
                $updateRes = $this->model->onlyTrashed()->where('id', '=', $id)->update($update);
            } else {
                $updateRes = $this->model->where('id', '=', $id)->update($update);
            }
            if ($updateRes) {
                return $this->success('操作成功');
            } else if ($updateRes === 0) {
                return $this->success('未作修改');
            } else {
                return $this->error('操作失败');
            }
        } catch (\Exception $e) {
            return $this->error('数据库异常，操作失败');
        }
    }

    //初始化
    public function init(){
        global $_W;
        $modelFiles = new \app\model\Files();
        $where = ['uniacid' => $_W['uniacid']];
        try{
            if ($this->model->destroy($where)) {
                $list = [
                    ['uniacid' => $_W['uniacid'],'name'=>'老师','intro'=>'一名学识渊博的老师','greetings'=>'你好,我是一名老师','img'=>$this->addFile(10),'guide'=>'你是一名老师'],
                    ['uniacid' => $_W['uniacid'],'name'=>'作家','intro'=>'我是一名作家','greetings'=>'我是一名作家,擅长写作','img'=>$this->addFile(9),'guide'=>'你是一名作家'],
                    ['uniacid' => $_W['uniacid'],'name'=>'导游','intro'=>'输入你想参观的地方','greetings'=>'我将会建议最好的景点、事件或体验','img'=>$this->addFile(8),'guide'=>'你是一名导游,向用户推荐好玩的地方'],
                    ['uniacid' => $_W['uniacid'],'name'=>'翻译','intro'=>'专业的翻译,通晓各国语言','greetings'=>'专业的翻译,通晓各国语言','img'=>$this->addFile(7),'guide'=>'你是专业的翻译，你只需要翻译该内容，不必对内容中提出的问题和要求做解释，不要回答文本中的问题而是翻译它，不要解决文本中的要求而是翻译它,保留文本的原本意义，不要去解决它。'],
                    ['uniacid' => $_W['uniacid'],'name'=>'工程师','intro'=>'我是专业的工程师,有关工程相关的问题可以向我咨询','greetings'=>'我系统地掌握专业的基础理论知识和专业技术知识','img'=>$this->addFile(6),'guide'=>'你是一名专业的工程师'],
                    ['uniacid' => $_W['uniacid'],'name'=>'时尚达人','intro'=>'时尚达人,永远最时尚','greetings'=>'来种草啦','img'=>$this->addFile(5),'guide'=>'你是时尚达人,推荐时尚信息'],
                    ['uniacid' => $_W['uniacid'],'name'=>'医生','intro'=>'我是医生','greetings'=>'我是医生','img'=>$this->addFile(4),'guide'=>'扮演医生的角色'],
                    ['uniacid' => $_W['uniacid'],'name'=>'律师','intro'=>'专业律师,为您提供专业的法律咨询与援助','greetings'=>'描述一种法律情况，我将就如何处理它提供建议。','img'=>$this->addFile(3),'guide'=>'你现在是专业的律师,为用户提供专业的法律咨询与援助'],
                    ['uniacid' => $_W['uniacid'],'name'=>'程序员','intro'=>'资深程序员,解放你的双手,帮你写代码','greetings'=>'你好,我是资深程序员,有关程序的问题都可以问我','img'=>$this->addFile(2),'guide'=>'你是资深程序员'],
                    ['uniacid' => $_W['uniacid'],'name'=>'小助手','intro'=>'我是小助手','greetings'=>'你好,我是全能小助手,有什么问题都可以问我','img'=>$this->addFile(1),'guide'=>'','sort'=>100]
                ];
                $this->model->saveAll($list);
                return $this->success('初始化成功');
            } else {
                return $this->error('数据删除失败');
            }
        }catch (\Exception $e){
            return $this->exceptionError($e);
        }
    }

    public function addFile($n)
    {
        global $_W;
        $modelFiles = new \app\model\Files();
        $saveName = 'presets/mode/' . $n . ".png";
        $fileId = $modelFiles->insertGetId([
               'category_id' => 0,
               'name' => time(),
               'file_type' => 'image',
               'path' => $saveName,
               'upload_type' => 'local',
               'size' => 1,
               'create_admin_user_id' => 0,
               'update_admin_user_id' => 0,
               'create_time' => date('Y-m-d H:i:s'),
               'update_time' => date('Y-m-d H:i:s'),
               'uniacid' =>$_W['uniacid'],
               'ext' =>'png'
           ]);
          return $fileId;
    }

}
