<?php

namespace app\controller\admin;

use app\service\api\UserServiceFacade;
use laytp\controller\Backend;
use laytp\library\CommonFun;
use laytp\library\UploadDomain;
use think\facade\Db;

/**
 * 会员管理
 */
class Words extends Backend
{
    protected $model;//当前模型对象
    protected $noNeedLogin = [];
    protected $noNeedAuth = [];

    protected function _initialize()
    {
        $this->model = new \app\model\Words();
    }

    //查看
    public function index()
    {
        global $_W;
        $where = $this->buildSearchParams();
        $where[] = ['uniacid','=',$_W['uniacid']];
        // $order = $this->buildOrder();
        $order = ['sort'=>'desc','id'=>'desc'];
        $data = $this->model->where($where)->order($order);
        $allData = $this->request->param('all_data');
        if ($allData) {
            $data = $data->select();
        } else {
            $limit = $this->request->param('limit', 10);
            $data = $data->paginate($limit)->toArray();
        }
        return $this->success('数据获取成功', $data);
    }

    //添加
    public function add()
    {
        global $_W;
        Db::startTrans();
        try {
            $post            = CommonFun::filterPostData($this->request->post());
            $post['uniacid'] = $_W['uniacid'];
            $result   = $this->model->save($post);
            if(!$result) throw new \Exception("添加失败");
            Db::commit();
            return $this->success('添加成功');
        } catch (\Exception $e) {
            Db::rollback();
            return $this->exceptionError($e);
        }
    }

    //查看详情
    public function info()
    {
        $id   = $this->request->param('id');
        $info = $this->model->findOrEmpty($id)->toArray();
        return $this->success('获取成功', $info);
    }

    //编辑
    public function edit()
    {
        Db::startTrans();
        try {
            $post = CommonFun::filterPostData($this->request->post());
            $navmenu = $this->model->findOrEmpty($post['id']);
            if (!$navmenu) throw new \Exception("id参数错误");
            $updateWords = $navmenu->update($post);
            if (!$updateWords) throw new \Exception("菜单基本信息保存失败");
            Db::commit();
            return $this->success('操作成功');
        } catch (\Exception $e) {
            Db::rollback();
            return $this->exceptionError($e);
        }
    }

    //删除
    public function del()
    {
        $ids = array_filter($this->request->param('ids'));
        if (!$ids) {
            return $this->error('参数ids不能为空');
        }
        try{
            if ($this->model->destroy($ids)) {
                return $this->success('数据删除成功');
            } else {
                return $this->error('数据删除失败');
            }
        }catch (\Exception $e){
            return $this->exceptionError($e);
        }
    }

    //设置状态
    public function setStatus()
    {
        $id       = $this->request->post('id');
        $fieldVal = $this->request->post('field_val');
        $isRecycle = $this->request->post('is_recycle');
        $update['status'] = $fieldVal;
        try {
            if($isRecycle) {
                $updateRes = $this->model->onlyTrashed()->where('id', '=', $id)->update($update);
            } else {
                $updateRes = $this->model->where('id', '=', $id)->update($update);
            }
            if ($updateRes) {
                return $this->success('操作成功');
            } else if ($updateRes === 0) {
                return $this->success('未作修改');
            } else {
                return $this->error('操作失败');
            }
        } catch (\Exception $e) {
            return $this->error('数据库异常，操作失败');
        }
    }

    //回收站
    public function recycle(){
        $where = $this->buildSearchParams();
        $order = $this->buildOrder();
        $limit = $this->request->param('limit', 10);
        $data  = $this->model->onlyTrashed()
                 ->order($order)->where($where)->paginate($limit)->toArray();
        return $this->success('回收站数据获取成功', $data);
    }

    //初始化
    public function init(){
        global $_W;
        $where = ['uniacid' => $_W['uniacid']];
        try{
            if ($this->model->destroy($where)) {
                $list = [
//                     ['uniacid' => $_W['uniacid'],'name'=>'日落时的城市天际线, 水彩墨风格','guide'=>'日落时的城市天际线, 水彩墨风格','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'一只猫坐在椅子上，戴着一副墨镜, 国潮','guide'=>'一只猫坐在椅子上，戴着一副墨镜, 国潮','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'巨大的纯白色城堡-油画,故障艺术','guide'=>'巨大的纯白色城堡-油画,故障艺术','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'Yggdrasil，世界树和地球融合在一起,故障艺术
// 荒岛,蒸汽波艺术','guide'=>'Yggdrasil，世界树和地球融合在一起,故障艺术
// 荒岛,蒸汽波艺术','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'戴着眼镜的猫，蒸汽波艺术','guide'=>'戴着眼镜的猫，蒸汽波艺术','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'只海鸥和史蒂文·西格正在进行一场凝视比赛，绘画,包豪斯','guide'=>'只海鸥和史蒂文·西格正在进行一场凝视比赛，绘画,包豪斯','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'梵高猫头鹰,包豪斯','guide'=>'梵高猫头鹰,包豪斯','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'4k专业HDR-DnD奇幻概念艺术小鸡施展幻觉咒语,概念艺术','guide'=>'4k专业HDR-DnD奇幻概念艺术小鸡施展幻觉咒语,概念艺术','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'年轻女孩，雪白皮肤，动漫脸，太阳光，看着你，镜头耀斑，杰作','guide'=>'年轻女孩，雪白皮肤，动漫脸，太阳光，看着你，镜头耀斑，杰作','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'动漫脸，氛围感，完美五官，精致眼睛，闪耀发光，风系魔法','guide'=>'动漫脸，氛围感，完美五官，精致眼睛，闪耀发光，风系魔法','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'汉服，旗袍，中国传统服饰，精致细节，动漫脸，水墨画','guide'=>'汉服，旗袍，中国传统服饰，精致细节，动漫脸，水墨画','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'两个人，帅气男孩，可爱女孩，pixiv，完美五官，美丽细节','guide'=>'两个人，帅气男孩，可爱女孩，pixiv，完美五官，美丽细节','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'猫，短毛猫，白毛，可爱，氛围感，精致眼睛，丰富细节','guide'=>'猫，短毛猫，白毛，可爱，氛围感，精致眼睛，丰富细节','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'小孩，可爱，光线感，完美五官，氛围感，白皮肤，二次元','guide'=>'小孩，可爱，光线感，完美五官，氛围感，白皮肤，二次元','type'=>'sentence'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'印象主义','guide'=>'印象主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'现实主义','guide'=>'现实主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'浪漫主义','guide'=>'浪漫主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'表现主义','guide'=>'表现主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'后印象主义','guide'=>'后印象主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'象征主义','guide'=>'象征主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'新艺术主义','guide'=>'新艺术主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'巴洛克风格','guide'=>'巴洛克风格','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'抽象表现主义','guide'=>'抽象表现主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'北欧文艺复兴','guide'=>'北欧文艺复兴','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'素人艺术，原始主义','guide'=>'素人艺术，原始主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'洛可可','guide'=>'洛可可','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'色域绘画','guide'=>'色域绘画','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'波普艺术','guide'=>'波普艺术','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'文艺复兴全盛期','guide'=>'文艺复兴全盛期','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'极简主义','guide'=>'极简主义','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'矫饰主义，文艺复兴晚期','guide'=>'矫饰主义，文艺复兴晚期','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'仰拍','guide'=>'仰拍','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'动态模糊','guide'=>'动态模糊','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'高反差','guide'=>'高反差','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'中心构图','guide'=>'中心构图','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'长曝光','guide'=>'长曝光','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'禅宗摄影','guide'=>'禅宗摄影','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'抽象微距镜头','guide'=>'抽象微距镜头','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'暗色调','guide'=>'暗色调','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'长时间曝光','guide'=>'长时间曝光','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'框架，取景','guide'=>'框架，取景','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'戴圆眼镜','guide'=>'戴圆眼镜','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'头戴皇冠','guide'=>'头戴皇冠','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'高质量','guide'=>'高质量','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'秀肌肉','guide'=>'秀肌肉','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'婚纱','guide'=>'婚纱','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'8k','guide'=>'8k','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'飞羽毛','guide'=>'飞羽毛','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'夜空','guide'=>'夜空','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'太阳光','guide'=>'太阳光','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'最佳画质','guide'=>'最佳画质','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'电影灯光','guide'=>'电影灯光','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'星空','guide'=>'星空','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'脸红','guide'=>'脸红','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'结婚','guide'=>'结婚','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'晴空','guide'=>'晴空','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'红色头发','guide'=>'红色头发','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'结婚','guide'=>'结婚','type'=>'word'],
//                     ['uniacid' => $_W['uniacid'],'name'=>'笑脸','guide'=>'笑脸','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'电吉他女孩','guide'=>'1girl, solo, black skirt, blue eyes, electric guitar, guitar, headphones, holding, holding plectrum, instrument, long hair, , music, one side up, teal hair, twin tails, playing guiter, pleated skirt, black shirt, indoors','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'长发女孩','guide'=>'best quality, ultra-detailed, illustration, intricate details, 8k, highres, extremely detailed_eyes,black eyes, long hair,Laughing_face,  braided_bangs, white hair, hair ribbon, (blush:0.5), upper body,(skyscraper:1.3),(ambient light:1.3), depth of field, easynegative','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'黑龙女','guide'=>'Original, illustrated, of the best quality, representative work, very detailed CGUnity8K wallpaper, color, IMID shooting 5, full body 5, dynamic angle, solo, bottle bottom, 1 young and cute girl 5 and Lolita, legendary lightning sword, detailed and beautiful eyes, beautiful face, glowing blue eyes//, silver pink gradient disheveled hair/,+perfect hand+1, air bangs, explosive lightning, perfect sword 1,+++lightning interwoven with sword/, Face expressionless, bottle bottom, purple flames around the wings behind the girl, Black Dragon King behind the girl','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'星夜','guide'=>'Panorama, oil painting, excellent image quality, extremely detailed CG unified 8k wallpaper, masterpiece, stunning dreamy and picturesque, dreamy and unreal, with blue pink clouds in the dreamy sky, some stars in the sky, a glassy lake, a large blue meteor crossing the sky and falling into the lake','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'花之冠冕','guide'=>'Petite, 1Girl, solo, white hair, hair strands, floating hair, headband, tiger teeth, Milky Way, necklace, center hem, torn clothes, awkward, by famous artist, beautiful, masterpiece, reflective hair, good lighting, looking at you, by Wadim Kashin, ray tracing, flowing hair, glossy hair, ultra delicate skin, masterpiece, masterpiece, detail, good lighting, glass tone, magnifying eyes, Reflective eyes, water eyes, girl portrait','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'水月镜花','guide'=>'The best lighting, the best shadows, an extremely exquisite and beautiful, only water, clear and blue-green sky, water as smooth as a mirror, petals, clouds connecting water and sky, colorful clouds, depth of field, beautiful detail sky, blue sky, beautiful detail clouds, beautiful detail water, reflection pool, extremely detailed CG unified 8k wallpaper, masterpiece, best quality, super details, dynamic angles, floating, Fine details, blooming, shining, twinkling stars, feathers, nature, sunshine, fairyland','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'女骑士','guide'=>'<Actual, complex details>, many red flowers, lying, pale skin, closed eyes, top view, hand on chest, holding bouquet, full armor, black knight armor, red pattern on armor, many details about armor, small chest, dynamic angle, floating hair, silver long hair, solo, 1 girl','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'赛博朋克风','guide'=>'((extremely fine CG unified 8k wallpaper)), ((extremely fine)), ((texture)), ((fine details)), (extremely fine and beautiful)), illustrations, lovely girls, white hair, tail, and wave high ponytail, robotic hands, looking at the audience, cyberpunk, cars, neon billboards','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'末日之城','guide'=>'Doomsday, epic reality, HDR, soft colors, apocalypse, abandoned, neutral colors, night, screen space refraction, complex details, intricate details, super details, art stations, movie shots, small episodes, complex backgrounds, turquoise and orange, special colors, flipping on the building','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'独角兽','guide'=>'awesome colorful illustration, unicorn, clip art style, lovely, cute, artstation','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'花之冠冕','guide'=>'Petite, 1Girl, solo, white hair, hair strands, floating hair, headband, tiger teeth, Milky Way, necklace, center hem, torn clothes, awkward, by famous artist, beautiful, masterpiece, reflective hair, good lighting, looking at you, by Wadim Kashin, ray tracing, flowing hair, glossy hair, ultra delicate skin, masterpiece, masterpiece, detail, good lighting, glass tone, magnifying eyes, Reflective eyes, water eyes, girl portrait','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'武士','guide'=>'Photo of 8k ultra realistic samurai warrior with pink war flag banner, full body, intricate pink and black armor, ornate cinematic lighting, high definition, vibrant illumination, trending on art station, hyperrealistic, high details, focused, unreal engine 5, dark, lumen reflections, depth of field, photography, reddish war torn environment, smoke and embers, cinematic','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'笼中鸟','guide'=>'Beautiful and detailed sky, beautiful and detailed sea, very detailed CGunity8k wallpaper, masterpiece, best quality, highly detailed, beautiful and detailed eyes, movie lighting, extremely exquisite and beautiful, highly detailed, masterpiece, best quality, super detailed, illustrations, disheveled hair, dynamic angle, a girl trapped in a huge glass bottle, whole body, drifting bottle, kneeling, reaching out, shackles, collar, arm rest, Gaze, overlook, panoramic, dreamy and picturesque, from above, glass','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'银发少女','guide'=>'(extremely detailed CG unity 8k wallpaper),(masterpiece), (best quality), (ultra-detailed), (best illustration),(best shadow), (an extremely delicate and beautiful), classic, dynamic angle, floating, finely detail, Depth of field, classic, (painting), (sketch), (bloom), (shine), glinting stars, a girl,solo,Winter clothing, diamond and glaring eyes, beautiful detailed cold face, very long blue and sliver hair, wavy hair, extremely delicate and beautiful girls, beautiful detailed eyes, glowing eyes,Upper body,(female focus), winter,
(forest),snow-covered landscape, (fairyland,flowers, nature),(sunlight),Hazy fog, mist','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'龙女','guide'=>'Masterpiece, premium, extremely thin CGunity8k wallpaper, movie angle, 1Girl, long hair, messy hair, beautiful and delicate eyes, beautiful and delicate hair, sitting, swords, torn clothes, blood on the face, sexy, silver belly armor, cowhorns, metal, complex patterns, cloak, nature, ruins, battlefield, flames, bravery, dark sky','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'维京国王','guide'=>'Close-up of Viking King emerging from wet black mud :: Dark metal helmet with two large black horns on the sides of the helmet :: dark warpaint :: scruffy black beard :: angry expression :: scar on face :: Viking Axe :: cinematic lighting, volumetric rays, moody, wet, dripping mud ','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'夕阳烈火','guide'=>'Masterpiece, illustrations, high-resolution, best quality, ultra detailed, depth of field, silhouette, bust, looking at the audience, 1Girl, solo, expressionless, bishopjo, beautiful and delicate red eyes, messy hair, black hair, middle breasts, hair between eyes, long hair, complex graphics, sharp black armor, sharp black shoulder armor, gorgeous shoulder armor, red shining rune details on armor, red torn cloak, ruins, Clouds, sunsets, light particles, breezes, embers, detailed background, surrounded by many beautiful and detailed red crystals, mountains','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'原神胡桃','guide'=>'((extremely fine CG unified 8k wallpaper)), ((extremely fine)), ((texture)), ((fine detail)), (extremely fine and beautiful)), Genshin impact, very close to the audience, ink, watercolor, spirit, happy girl, black hexagonal cap, black beret, flying hair, curly hair, beautiful detail eyes, cherry blossom pupils, burning forest, sparks, animated face, black dress, HuTao, messy long hair, Chinese brides dress, token, brown black hair, flame, Depth of Field, Ruby Necklace, Fire Butterfly, Flying Sparks, Yin Yang Master, Ghost, Revealing Teeth, Smiling','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'赛博都市','guide'=>'Reunion of man, team, squad, cyberpunk, abstract, full hd render + 3d octane render +4k UHD + immense detail + dramatic lighting + well lit + black, purple, blue, pink, cerulean, teal, metallic colours, + fine details + octane render + 8k','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'虚空','guide'=>'Venom and Carnage fight, character, realistic body, realistic muscle cyberpunk, glowing fire, film lens, octane render, 8K','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'毒液','guide'=>'Venom and Carnage fight, character, realistic body, realistic muscle cyberpunk, glowing fire, film lens, octane render, 8K','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'夏日的烟花','guide'=>'Beautiful and meticulous girl, extremely meticulous eyes and face, beautiful and meticulous eyes, absurd, incredible absurdity, huge file size, super detailed, Hercules, very detailed, of the best quality, masterpiece, illustration, an extremely exquisite and beautiful, very detailed, CG, unity, 8K wallpaper, surprising, exquisite details, masterpiece, the best quality, official art, extremely detailed CGUnity8K wallpaper, Iron Tree, Honeysuckle, Summer Festival, Sacrificial Ceremonies, Watching Fireworks, Back to Camera, Hand in Hand, Panorama, Two People Leaning Together, Black haired Girl, Black haired Boy, Beautiful and Meticulous Girl, Extremely Meticulous Eyes and Face, Beautiful and Meticulous Eyes, Absurd, Unbelievable Absurdity, Huge File Size, Super Detailed, Hercules, Very Detailed, Best Quality, Masterpiece, Example, An Extremely Exquisite and Beautiful, Very Detailed, CG, Unity, 8K Wallpaper, Amazing, Fine Details, Masterpiece, Best Quality, Official Art, Extremely Detailed CGUnity8K Wallpaper, Iron Tree Honeysuckle, Summer Festival, Sacrificial Ceremonies, Watching Fireworks, Back to Camera, Hand in Hand, Panorama, Two People Leaning Together, Black haired Girl, Black haired Boy','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'森林精灵','guide'=>'Ultra detailed, illustrated, extremely exquisite and beautiful, dynamic angle, floating, beautiful and meticulous eyes, detailed light 1 girl, loli, floating hair, shining eyes, pointed ears, white hair, green eyes, backless dresses, feathers, leaves, nature, sunlight, rivers, forests, painting, sketching, blooming','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'末日机甲','guide'=>'Blasphemy, masterpiece, absurdity, realism, color difference, science fiction, mechanized transformation, extremely fine CG, impossible clothing, 1Girl, mature women, combat posture, mechanical joints, super machines, giant breasts, narrow waist, wide hips, thick thighs, long legs, mechanical armor, glass armor, black and white bronze clothing, torn clothing, damaged clothing, glowing clothing, detailed machinery, detailed structure, mechanical mask, mechanical neck protection, mechanical shoulder armor, Mechanical arm guards, mechanical wrist guards, armored boots, panels, mechanical spheres, lightning balls, searchlights, handheld weapons, harsh sky, harsh atmosphere, cannons, neon light blades, city appearance, damaged buildings, monsters, mysterious creatures, wide angle','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'玫瑰男','guide'=>'{{{solo}}},Extremely detailed CG illustration of a beautiful, wounded young man with short, spiky hair, surrounded by a sea of roses, (best quality:1.5), (close-up), (masterpiece:1.8), (1 boy:1.2), (sad expression:1.5), (deep scars:1.4), (old wounds:1.3), (healing cuts:1.3), (flowers:1.5), (romantic setting:1.5), (dramatic lighting:1.3), (rich colors:1.3), (handsome face:1.5), (delicate features:1.3), (fragile demeanor:1.3), (piercing eyes:1.2), (sharp nose:1.2), (chiseled jawline:1.2), (defined cheekbones:1.2), (full lips:1.2), (tattered clothes:1.3), (blood stains:1.3), (dried blood:1.2), (worn out shoes:1.2), (broken jewelry:1.2)','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'校服','guide'=>'asterpiece, best quality, salute, hand on hip, full body, 1girl, bangs, black choker, black necktie, blonde hair, blue skirt, blush, bracelet, breasts, choker, clothes around waist, collarbone, collared shirt, cowboy shot, dress shirt, ear piercing, eyebrows visible through hair, gradient hair, grin, gyaru, jewelry, kogal, long hair, looking at viewer, loose necktie, necktie, piercing, plaid, plaid skirt, pleated skirt, red eyes, ring, school uniform, shirt, skirt, smile, solo, white shirt, street, sky, cherry blossoms, petals','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'森林精灵','guide'=>'Ultra detailed, illustrated, extremely exquisite and beautiful, dynamic angle, floating, beautiful and meticulous eyes, detailed light 1 girl, loli, floating hair, shining eyes, pointed ears, white hair, green eyes, backless dresses, feathers, leaves, nature, sunlight, rivers, forests, painting, sketching, blooming','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'室内场景','guide'=>'a home built in a huge Soap bubble, windows, doors, porches, awnings, middle of SPACE, cyberpunk lights, Hyper Detail, 8K, HD, Octane Rendering, Unreal Engine, V-Ray, full hd','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'废土','guide'=>'Masterpiece, top-notch quality, ultra fine illustrations, dark crystals on the ground, huge fire clouds, ruins, broken mountains, nature, painting, very exquisite 8KCG wallpaper, meteorites crossing the sky, the end of the world, from the side','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'星云兔','guide'=>'galaxies, spirals, space, nebulae, stars, smoke, iridescent, intricate detail, in the shape of a rabbit, octane render, 8k','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'幻想物语','guide'=>'Beautiful details, extremely detailed eyes and face, beautiful and detailed eyes, detailed lighting, movie lighting, lens flares, leaks, 3D, sunlight, illumination, beautiful and detailed glow, absurdity, incredible absurdity, huge file size, super detail, Hercules, very detailed, dynamic angle, floating, embellishment, painting, depth of field, depth of field, nature, best quality, masterpiece, illustration, An extremely exquisite and beautiful, very detailed, CG, Unity, 8K wallpaper, surprising, exquisite details, masterpiece, the best quality, official art, extremely detailed CGUnity8K wallpaper, iron tree and honeysuckle, starry sky, star cluster, starry sky, twinkling stars, panoramic view, two people leaning together, black haired girl, black haired boy, West Cloud Country Story, sitting on the water, romantic, dynamic angle, fantasy, hair shining, Dramatic angle, black highlight, rich color 5, with many gorgeous tones, floating and falling petals','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'克鲁苏章鱼','guide'=>'Masterpiece, best quality, official art, very detailed CGUnity8K wallpaper, very detailed, illustrations, reality, ocean, Cthulhu, octopus, black wings, chest, muscles, horn, ancient, dark, night, bust, monster, blue eyes, mouth, big eyes','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'江南','guide'=>'Very detailed CGUnity8K wallpaper, masterpiece, best quality, ultra detailed, best illustration, best shadow, very exquisite and beautiful, dynamic angle, floating, fairyland, dynamic angle, ocean of flowers, beautiful and detailed garden, wind, classic, spring, detailed lighting, feathers, cherry blossoms, nature, sunshine, river, forest, floating palace, best architecture, beautiful and exquisite water, painting, sketch, bloom, shining','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'夕阳烈火','guide'=>'Masterpiece, illustrations, high-resolution, best quality, ultra detailed, depth of field, silhouette, bust, looking at the audience, 1Girl, solo, expressionless, bishopjo, beautiful and delicate red eyes, messy hair, black hair, middle breasts, hair between eyes, long hair, complex graphics, sharp black armor, sharp black shoulder armor, gorgeous shoulder armor, red shining rune details on armor, red torn cloak, ruins, Clouds, sunsets, light particles, breezes, embers, detailed background, surrounded by many beautiful and detailed red crystals, mountains','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'国风女孩','guide'=>'Beautiful face, hair ornament, solo,looking at viewer,smile,closed mouth,lips china dress,dress,hair ornament, necklace, jewelry, long hair, earrings, chinese clothes, architecture,east asian architecture,building,outdoors,rooftop,city,cityscape','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'樱花树屋','guide'=>'THE CHERRY BLOSSOM TREE HOUSE :: beautiful ornate treehouse in a gigantic pink cherry blossom tree :: on a high blue grey and brown cliff with light snow and pink cherry blossom trees :: Roger Deakins and Moebius and Alphonse Much and Guweiz :: Intricate details, very realistic, cinematic lighting, volumetric lighting, photographic, bokeh defocus dof','type'=>'word'],
                    ['uniacid' => $_W['uniacid'],'name'=>'风暴船','guide'=>'A real photographic landscape painting with incomparable reality,Super wide,Ominous sky,Sailing boat,Wooden boat,Lotus,Huge waves,Starry night,Harry potter,Volumetric lighting,Clearing,Realistic,James gurney,artstation','type'=>'word'],
                ];
                $this->model->saveAll($list);
                return $this->success('初始化成功');
            } else {
                return $this->error('数据删除失败');
            }
        }catch (\Exception $e){
            return $this->exceptionError($e);
        }
        // return $this->success('获取成功', $_W);
    }
}
