<?php

namespace app\service\api;

use app\service\ConfServiceFacade;
use laytp\traits\Error;
use think\facade\Cache;
use think\facade\Config;
use think\facade\Request;
use EasyWeChat\Factory;

class Pay{
  use Error;

  /**
     * phpmailer对象
     */
     protected $wechat;
     protected $pay;


    /**
     * 构造函数
     */
    public function __construct()
    {
        $wechat = ConfServiceFacade::groupGet('system.wechat', true);
        // print_r($wechat);
        $pay = ConfServiceFacade::groupGet('system.pay', true);
        $this->wechat = $wechat;
        $this->pay = $pay;
    }


  /**
   * 初始化
   * @param $token
   * @return bool
   * @throws \think\db\exception\DataNotFoundException
   * @throws \think\db\exception\DbException
   * @throws \think\db\exception\ModelNotFoundException
   */
  public function option($plateform='wechat')
  {
    $wechat = ConfServiceFacade::groupGet('system.'.$plateform, true);
    // print_r($wechat);
    $pay = ConfServiceFacade::groupGet('system.pay', true);
    // print_r(IA_ROOT_WK.'/public/static/storage/'.$pay['cert_filename']);
    if(!empty($pay['cert_filename'])){
        $config = [
            // 必要配置
            'app_id'             => $wechat['appid'],
            'mch_id'             => $pay['wechat_pay_mchid'],
            'key'                => $pay['wechat_pay_apikey'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)

            // 如需使用敏感接口（如退款、发送红包等）需要配置 API 证书路径(登录商户平台下载 API 证书)
            'cert_path'          => IA_ROOT_WK.'/public/static/storage/'.$pay['cert_filename'], // XXX: 绝对路径！！！！
            'key_path'           => IA_ROOT_WK.'/public/static/storage/'.$pay['key_filename'],      // XXX: 绝对路径！！！

            'notify_url'         => request()->domain() . SURL . '/index.php/api.pay/notify',     // 你也可以在下单时单独设置来想覆盖它

          //   'log' => [
          //     'level' => 'debug',
          //     'file' => __DIR__.'/pay.log',
          // ],
        ];
    } else{
        $config = [
            // 必要配置
            'app_id'             => $wechat['appid'],
            'mch_id'             => $pay['wechat_pay_mchid'],
            'key'                => $pay['wechat_pay_apikey'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)

            'notify_url'         => request()->domain() . SURL . '/index.php/api.pay/notify',     // 你也可以在下单时单独设置来想覆盖它

          //   'log' => [
          //     'level' => 'debug',
          //     'file' => __DIR__.'/pay.log',
          // ],
        ];
    }


    return Factory::payment($config);
  }


  /**
     * 付款到微信零钱
     * sOpenid：收款方openid
     * nMoney：转账金额
     */
    public function transfer_batches($sOpenid, $nMoney)
    {
        //请求URL
       $url = 'https://api.mch.weixin.qq.com/v3/transfer/batches';
       //请求方式
       $http_method = 'POST';
       $sOrderId = 'wd3452'.time();
       //请求参数
       $data = [
           'appid' => $this->wechat['appid'],//申请商户号的appid或商户号绑定的appid（企业号corpid即为此appid）
           'out_batch_no' =>$sOrderId,//商户系统内部的商家批次单号，要求此参数只能由数字、大小写字母组成，在商户系统内部唯一
           'batch_name' => '提现至微信零钱',//该笔批量转账的名称
           'batch_remark' => '提现至微信零钱',//转账说明，UTF8编码，最多允许32个字符
           'total_amount' => $nMoney,//转账金额单位为“分”。转账总金额必须与批次内所有明细转账金额之和保持一致，否则无法发起转账操作
           'total_num' => 1,//一个转账批次单最多发起三千笔转账。转账总笔数必须与批次内所有明细之和保持一致，否则无法发起转账操作
           'transfer_detail_list' => [
               [//发起批量转账的明细列表，最多三千笔
                   'out_detail_no' => $sOrderId,//商户系统内部区分转账批次单下不同转账明细单的唯一标识，要求此参数只能由数字、大小写字母组成
                   'transfer_amount' => $nMoney,//转账金额单位为分
                   'transfer_remark' => '提现至微信零钱',//单条转账备注（微信用户会收到该备注），UTF8编码，最多允许32个字符
                   'openid' => $sOpenid,//openid是微信用户在公众号appid下的唯一用户标识（appid不同，则获取到的openid就不同），可用于永久标记一个用户
               ]]
       ];
       if ($nMoney >= 2000) {
           // if (empty($withdrawApply['real_name'])) {
           //     throw new \Exception('转账金额 >= 2000元，收款用户真实姓名必填');
           // }
           // $data['transfer_detail_list'][0]['user_name'] = self::getEncrypt($withdrawApply['real_name'],$config);
       }

       $token  = self::token($url,$http_method,$data,$this->pay);//获取token
       $result = self::https_request($url,json_encode($data),$token);//发送请求
       $result_arr = json_decode($result,true);

       if(!isset($result_arr['create_time'])) {//批次受理失败
           throw new \Exception($result_arr['message']);
       }

       //批次受理成功，进行操作


       return true;
    }

    /**
     * @notes 获取签名
     */
    public static function token($url,$http_method,$data,$config)
    {
        $timestamp   = time();//请求时间戳
        $url_parts   = parse_url($url);//获取请求的绝对URL
        $nonce       = $timestamp.rand('10000','99999');//请求随机串
        $body        = empty($data) ? '' : json_encode((object)$data);//请求报文主体
        $stream_opts = [
            "ssl" => [
                "verify_peer"=>false,
                "verify_peer_name"=>false,
            ]
        ];
        $apiclient_cert_arr = openssl_x509_parse(file_get_contents(IA_ROOT_WK.'/public/static/storage/'.$config['cert_filename'],false, stream_context_create($stream_opts)));
        $serial_no          = $apiclient_cert_arr['serialNumberHex'];//证书序列号
        $mch_private_key    = file_get_contents(IA_ROOT_WK.'/public/static/storage/'.$config['key_filename'],false, stream_context_create($stream_opts));//密钥
        $merchant_id = $config['wechat_pay_mchid'];//商户id
        $canonical_url = ($url_parts['path'] . (!empty($url_parts['query']) ? "?${url_parts['query']}" : ""));
        $message = $http_method."\n".
            $canonical_url."\n".
            $timestamp."\n".
            $nonce."\n".
            $body."\n";

        openssl_sign($message, $raw_sign, $mch_private_key, 'sha256WithRSAEncryption');
        $sign = base64_encode($raw_sign);//签名
        $schema = 'WECHATPAY2-SHA256-RSA2048';
        $token = sprintf('mchid="%s",nonce_str="%s",timestamp="%d",serial_no="%s",signature="%s"',
            $merchant_id, $nonce, $timestamp, $serial_no, $sign);//微信返回token

        return $schema.' '.$token;
    }

    /**
     * @notes 发送请求
     */
    public static function https_request($url,$data,$token)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, (string)$url);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, FALSE);
        if (!empty($data)){
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        //添加请求头
        $headers = [
            'Authorization:'.$token,
            'Accept: application/json',
            'Content-Type: application/json; charset=utf-8',
            'User-Agent:Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36',
        ];
        if(!empty($headers)){
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        }
        $output = curl_exec($curl);
        curl_close($curl);
        return $output;
    }

    /**
     * @notes 敏感信息加解密
     */
    public static function getEncrypt($str,$config)
    {
        //$str是待加密字符串
        $public_key = file_get_contents(IA_ROOT_WK.'/public/static/storage/'.$this->pay['cert_filename']);
        $encrypted = '';
        if (openssl_public_encrypt($str, $encrypted, $public_key, OPENSSL_PKCS1_OAEP_PADDING)) {
            //base64编码
            $sign = base64_encode($encrypted);
        } else {
            throw new \Exception('encrypt failed');
        }
        return $sign;
    }

    /**
     * @notes 商家明细单号查询
     */
    public static function detailsQuery($withdrawApply,$config)
    {
        //请求URL
        $url = 'https://api.mch.weixin.qq.com/v3/transfer/batches/out-batch-no/'.$withdrawApply['batch_no'].'/details/out-detail-no/'.$withdrawApply['sn'];
        //请求方式
        $http_method = 'GET';
        //请求参数
        $data = [];
        $token  = self::token($url,$http_method,$data,$config);//获取token
        $result = self::https_request($url,$data,$token);//发送请求
        $result_arr = json_decode($result,true);
        return $result_arr;
    }

}
