<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Http;

use W7\Sdk\Module\Interfaces\UriInterface;
use W7\Sdk\Module\Support\Utils;

/**
 * http client
 */
class Client
{
    /** @var Uri */
    protected $base_uri;

    /** @var CurlFactory */
    protected $factory;

    protected $config = [];

    public function __construct($base_uri, $config = [])
    {
        $this->base_uri = Utils::uriFor($base_uri);
        if (!function_exists('curl_exec')) {
            throw new \RuntimeException('curl extension is required');
        }
        $this->factory = new CurlFactory(3);
        $this->config  = $config;
    }

    public function post($uri = '', array $options = [])
    {
        return $this->request('POST', $uri, $options);
    }

    public function get($uri = '', array $options = [])
    {
        return $this->request('GET', $uri, $options);
    }

    protected function request($method, $uri = '', array $options = [])
    {
        $headers = (isset($options['headers']) ? $options['headers'] : []);
        $body    = (isset($options['body']) ? $options['body'] : []);
        $version = (isset($options['version']) ? $options['version'] : '1.1');
        if (is_array($body)) {
            $body                    = \http_build_query($body, '', '&');
            $headers['Content-Type'] = 'application/x-www-form-urlencoded';
        }
        $uri     = $this->buildUri(Utils::uriFor($uri));
        $request = new Request($method, $uri, $headers, $body, $version);
        if (isset($this->config['request_middlewares'])) {
            $request = $this->config['request_middlewares']->handler($request);
            unset($this->config['request_middlewares']);
        }

        unset($options['headers'], $options['body'], $options['version']);
        $handler = $this->factory->create($request, $options + $this->config);
        \curl_exec($handler->handle);
        $handler->errno = \curl_errno($handler->handle);
        return CurlFactory::finish($handler, $this->factory);
    }

    private function buildUri(UriInterface $uri)
    {
        if (isset($this->base_uri)) {
            $uri = UriResolver::resolve($this->base_uri, $uri);
        }

        return '' === $uri->getScheme() && '' !== $uri->getHost() ? $uri->withScheme('http') : $uri;
    }
}
