<?php

namespace W7\Sdk\Module\Http;

use W7\Sdk\Module\Support\Utils;

final class HeaderProcessor
{
    public $version;

    public $status;

    public $reason;

    public $headers;

    public function __construct($version, $status, $reason, $headers)
    {
        $this->version = $version;
        $this->status = $status;
        $this->reason = $reason;
        $this->headers = $headers;
    }

    /**
     * 返回HTTP版本、状态代码、原因短语和协议头信息。
     *
     * @param string[] $headers
     *
     * @return static
     *
     * @throws \RuntimeException
     */
    public static function parseHeaders(array $headers)
    {
        if ($headers === []) {
            throw new \RuntimeException('Expected a non-empty array of header data');
        }

        $parts = \explode(' ', \array_shift($headers), 3);
        $version = \explode('/', $parts[0]);
        $version = (isset($version[1])) ? $version[1] : null;

        if ($version === null) {
            throw new \RuntimeException('HTTP version missing from header data');
        }

        $status = (isset($parts[1]))  ? $parts[1] : null;


        if ($status === null) {
            throw new \RuntimeException('HTTP status code missing from header data');
        }

        $reason = (isset($parts[2])) ? $parts[2] : null;

        return new HeaderProcessor($version, (int) $status, $reason, Utils::headersFromLines($headers));
    }
}
