<?php

namespace W7\Sdk\Module\Http;

use W7\Sdk\Module\Interfaces\RequestInterface;
use W7\Sdk\Module\Interfaces\ResponseInterface;
use W7\Sdk\Module\Interfaces\StreamInterface;
use W7\Sdk\Module\Middlewares\ResponseMiddlewares;
use W7\Sdk\Module\Support\Utils;

final class ResponseHandle
{
    /**
     * @var resource|\CurlHandle Curl 资源
     */
    public $handle;

    /**
     * @var StreamInterface 数据写入的地方
     */
    public $sink;

    /**
     * @var array 收到的 Http 协议头
     */
    public $headers = [];

    /**
     * @var ResponseInterface|null 收到的响应
     */
    public $response;

    /**
     * @var RequestInterface 正在发送的请求
     */
    public $request;

    /**
     * @var array 请求参数
     */
    public $options = [];

    /**
     * @var int cURL 错误代码
     */
    public $errno = 0;

    /**
     * @var \Throwable|null 获取协议头期间出现的异常
     */
    public $onHeadersException;

    /**
     * @var \Exception|null 在createResponse过程中出现的异常
     */
    public $createResponseException;

    /** @var ResponseMiddlewares */
    public $responseMiddlewares;

    /**
     * 根据请求参数创建一个 ResponseHandle
     *
     * @throws \RuntimeException 如果没有收到协议头或第一个协议头行无效。
    */
    public function createResponse()
    {
        $headersInfo = HeaderProcessor::parseHeaders($this->headers);
        $headers = $headersInfo->headers;

        $normalizedKeys = Utils::normalizeHeaderKeys($headers);

        if (!empty($this->options['decode_content']) && isset($normalizedKeys['content-encoding'])) {
            $headers['x-encoded-content-encoding'] = $headers[$normalizedKeys['content-encoding']];
            unset($headers[$normalizedKeys['content-encoding']]);
            if (isset($normalizedKeys['content-length'])) {
                $headers['x-encoded-content-length'] = $headers[$normalizedKeys['content-length']];

                $bodyLength = (int) $this->sink->getSize();
                if ($bodyLength) {
                    $headers[$normalizedKeys['content-length']] = $bodyLength;
                } else {
                    unset($headers[$normalizedKeys['content-length']]);
                }
            }
        }

        // 将响应与解析后的标头附加到Response Handle句柄。
        $this->response = new Response(
            $headersInfo->status,
            $headers,
            $this->sink,
            $headersInfo->version,
            $headersInfo->reason
        );
    }

    /**
     * @param string $name
     *
     * @return void
     *
     * @throws \BadMethodCallException
     */
    public function __get($name)
    {
        $msg = $name === 'handle' ? 'The ResponseHandle has been released' : 'Invalid property: '.$name;
        throw new \BadMethodCallException($msg);
    }
}
