<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Middlewares;

use W7\Sdk\Module\Interfaces\RequestInterface;
use W7\Sdk\Module\Support\Utils;

class RequestMiddlewares
{
    protected $appId;

    protected $appSecret;

    protected $linkAppId;

    protected $accountType;

    public function __construct($app_id, $app_secret, $link_app_id, $account_type)
    {
        $this->appId       = $app_id;
        $this->appSecret   = $app_secret;
        $this->linkAppId   = $link_app_id;
        $this->accountType = $account_type;
    }

    public function handler(RequestInterface $request)
    {
        $app_id       = $this->appId;
        $app_secret   = $this->appSecret;
        $link_app_id  = $this->linkAppId;
        $account_type = $this->accountType;

        $timeStamp  = time();
        $nonce      = $this->random(16);
        $bodyString = $request->getBody()->getContents();
        $body       = json_decode($bodyString, true);

        if (JSON_ERROR_NONE !== json_last_error()) {
            $request->getBody()->rewind();
            parse_str($request->getBody()->getContents(), $body);
        }

        if ('POST' === $request->getMethod()) {
            $request = $request->withHeader('Content-Type', 'application/x-www-form-urlencoded');
        }

        $data = [
            'appid'       => $app_id,
            'link_app_id' => $link_app_id,
            'timestamp'   => $timeStamp,
            'type'        => $account_type,
            'nonce'       => $nonce,
        ];

        if (!empty($body)) {
            $data['body'] = $body;
        }

        $data['sign'] = $this->getSign($data, $app_secret);

        $bodyStream = Utils::streamFor(http_build_query($data));
        return $request->withBody($bodyStream);
    }

    protected function getSign($data, $app_secret = '')
    {
        unset($data['sign']);

        ksort($data, SORT_STRING);

        return md5(http_build_query($data, '', '&') . $app_secret);
    }

    protected function random($length = 16)
    {
        $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $str   = '';
        for ($i = 0; $i < $length; $i++) {
            $str .= $chars[mt_rand(0, strlen($chars) - 1)];
        }

        return $str;
    }
}
