<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Pay;

use W7\Sdk\Module\Exceptions\ApiException;
use W7\Sdk\Module\Exceptions\ApiHttpException;
use W7\Sdk\Module\Exceptions\ConnectException;
use W7\Sdk\Module\Exceptions\RequestException;
use W7\Sdk\Module\Interfaces\ResponseInterface;
use W7\Sdk\Module\Support\ApiResponse;

class Wechat extends BasePay
{
    /** @var string  */
    protected $type = 'wechat';

    /**
     * 申请退款
     *
     * @param string $out_refund_no  商户系统内部的退款单号，商户系统内部唯一，只能是数字、大小写字母_-|*@ ，同一退款单号多次请求只退一笔。
     * @param int    $refund         退款金额，单位为分，只能为整数，不能超过原订单支付金额。
     * @param int    $total          原支付交易的订单总金额，单位为分，只能为整数。
     * @param string $transaction_id 原支付交易对应的微信订单号
     * @param string $out_trade_no   原支付交易对应的商户订单号
     * @param array  $other          其他非必填参数
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     */
    public function refund(
        $out_refund_no,
        $refund,
        $total,
        $transaction_id = '',
        $out_trade_no = '',
        array $other = []
    ) {
        return $this->request(
            'refund',
            compact(
                'out_refund_no',
                'out_trade_no',
                'refund',
                'total',
                'transaction_id',
                'out_trade_no',
                'other'
            )
        );
    }

    /**
     * JSAPI下单
     *
     * @param string $description  商品描述
     * @param string $out_trade_no 商户系统内部订单号，只能是数字、大小写字母_-*且在同一个商户号下唯一
     * @param int    $total        订单总金额，单位为分。
     * @param string $openid       用户在直连商户appid下的唯一标识。
     * @param array  $other        其他非必填参数
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     */
    public function payTransactionsJsapi(
        $description,
        $out_trade_no,
        $total,
        $openid,
        array $other = []
    ) {
        return $this->request(
            'payTransactionsJsapi',
            compact(
                'description',
                'out_trade_no',
                'total',
                'openid',
                'other'
            )
        );
    }

    /**
     * Native下单
     *
     * @param string $description  商品描述
     * @param string $out_trade_no 商户系统内部订单号，只能是数字、大小写字母_-*且在同一个商户号下唯一
     * @param int    $total        订单总金额，单位为分。
     * @param array  $other        其他非必填参数
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     */
    public function payTransactionsNative(
        $description,
        $out_trade_no,
        $total,
        array $other = []
    ) {
        return $this->request(
            'payTransactionsNative',
            compact(
                'description',
                'out_trade_no',
                'total',
                'other'
            )
        );
    }
}
