<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Support;

use W7\Sdk\Module\Exceptions\ApiException;
use W7\Sdk\Module\Exceptions\ApiHttpException;
use W7\Sdk\Module\Exceptions\ConnectException;
use W7\Sdk\Module\Exceptions\RequestException;
use W7\Sdk\Module\Http\Client;
use W7\Sdk\Module\Interfaces\ResponseInterface;

class AccountRequest implements ApiRequest
{
    /** @var Client */
    protected $client;

    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function getAccessToken()
    {
        return $this->client->post('open/api/account/getAccessToken');
    }

    /**
     * @param string $js_code
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function jsCode2Session($js_code)
    {
        return $this->client->post('open/api/account/jsCode2Session', [
            'form_params' => [
                'code' => $js_code,
            ],
        ]);
    }

    /**
     * @param string $code
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function snsOauthAccessToken($code)
    {
        return $this->client->post('open/api/account/snsOauthAccessToken', [
            'form_params' => [
                'code' => $code,
            ],
        ]);
    }
}
